//*****************************************************************************
// Projeto  : Teste RingComm com placa PE-STM32 by Arne
//            Permite testar a comunicação em anel via RS232 com outros
//            dispositivos/microcontroladores/microprocessadores
//
// Arquivo  : main.c
// Autor    : Eng. Márcio José Soares
// Versão   : V1.0
// Data     : 18/05/2023
// Descrição: Estudos com STM32F103C8T6 através da placa de estudos PE-STM32
//
// IDE      : SW4STM32 (Eclipse - Linux)
// Gravador : STLink V2
// Gravação : ST-FLASH (Linux)
//
//*****************************************************************************
// Disposição dos pinos:
//
//	Port A:
//		PA0	 - D4   - dado bit4 LCD
//		PA1  - D5   - dado bit5 LCD
//		PA2  - D6   - dado bit6 LCD
//		PA3  - D7   - dado bit7 LCD
//      PA8  - BTN1 - envia mensagem para dispositivo 2
//      PA9  - BTN4 - envia mensagem para dispositivo 3
//      PA10 - BTN7 - envia mensagem para dispositivo 4
//
//	Port B:
//		PB5  - LED1
//		PB6  - LED2
//		PB7  - LED3
//		PB8  - LED4
//		PB10 - TX3  - RX 	- USART3
//      PB11 - RX3  - TX 	- USART3
//		PB13 - RS   - LCD
//		PB14 - RW   - LCD
//		PB15 - EN   - LCD
//
//  Port C:
//		PC13 - LED  - Live
//
//*****************************************************************************
// Últimas alterações:
//	em 18/05/2023:
//		- criado esse projeto
//
//  em 23/05/2023:
//		- inicio dos testes com Arduino e PIC
//      - inserido saída por timeout
//		- inserido verificação de erro no nr do dispositivo
//        para evitar mensagem se repetindo
//
//  em 25/05/2023:
//		- testes para primeira versão ok!!!
//   	- divulgando no site e no canal!
//*****************************************************************************

//*****************************************************************************
// Arquivos incluídos
//*****************************************************************************
#include "main.h"

//*****************************************************************************
// Funções do módulo
//*****************************************************************************

//*******************************************************************************
// Init pins - configura pinos para uso
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void init_pins(void){

	GPIO_InitTypeDef GPIO_InitStructMain;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);

	GPIO_InitStructMain.GPIO_Pin = LED_Live;
	GPIO_InitStructMain.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructMain.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(PORTLED, &GPIO_InitStructMain);

	GPIO_InitStructMain.GPIO_Pin = LED1 | LED2 | LED3 | LED4;
	GPIO_InitStructMain.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructMain.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(PORTLED2, &GPIO_InitStructMain);

	GPIO_InitStructMain.GPIO_Pin = BTN1 | BTN4 | BTN7;
	GPIO_InitStructMain.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_InitStructMain.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(PORTBTN, &GPIO_InitStructMain);

	GPIO_ResetBits(PORTLED, LED_Live);

}

//*******************************************************************************
// Init vars - inicia variáveis
//
// Entradas - nenhuma
// Saídas   - nenhuma
//*******************************************************************************
void init_vars(void){

	LED_live = 0;
	autoriza = 0;
	time500ms = 0;

	counter  = 0;
	prima = 0;
	envio = 0;

	mytimeout = 0;
	timeout_aut = 0;

#ifdef __DEBUG__
	debug = 0x30;
#endif
}

//*******************************************************************************
// getBTNs - lê chaves
//
// Entradas - nenhuma
// Saídas   - nr do terminal a enviar
//*******************************************************************************
uint8_t getBTNs(void){

	uint8_t ret = 0;

	if(!readBTN1){								// pediu para enviar mensagem para terminal 2 - PIC
		while(!readBTN1)
			DelayMs(5);
		ret = 0x01;
		setLED1;
		resLED2;
		resLED3;
	}
	else if(!readBTN4){							// pediu para enviar mensagem para terminal 3 - Arduino
		while(!readBTN4)
			DelayMs(5);
		ret = 0x02;
		resLED1;
		setLED2;
		resLED3;
	}
	else if(!readBTN7){							// pediu para enviar mensagem para terminal 4 - placa Z80 1990
		while(!readBTN7)
			DelayMs(5);
		ret = 0x04;
		resLED1;
		resLED2;
		setLED3;
	}

	return(ret);
}

//*******************************************************************************
// Função main - função principal
//
// Entradas - nenhuma
// Saídas   - nenhuma (zero apenas)
//*******************************************************************************
int main(void){

	char RxTerm[3];							    // var para inserir dados recebidos do terminal
	char dados[TAMDATA];

	uint8_t mhigh, mlow;
	uint8_t toTerm = 0;
	uint16_t msgcounter = 0;

	init_pins();								// inicia pinos LEDs e botões de entrada
	init_vars();								// inicia variáveis globais
	DelayInit();								// inicia delay

	usart3_init();								// inicia usart3- 9600

	resLED1;									// LEDs apagados
	resLED2;
	resLED3;
	resLED4;

	DelayMs(1000);								// aguarda LCD iniciar

	//configura LCD
	lcd16x2_init(LCD16X2_DISPLAY_ON_CURSOR_ON_BLINK_OFF);	//inicia LCD

	//envia msg via lcd
	lcd16x2_clrscr();
	//envia msg via lcd
	lcd16x2_clrscr();
	lcd16x2_gotoxy(2,0);						// posiciona na linha 1, coluna 2
	lcd16x2_puts("BEM-VINDO AO");				// escreve a partir da última posição
	lcd16x2_gotoxy(2,1);						// posiciona na linha 2, coluna 2
	lcd16x2_puts("LAB do Arne");				// escreve a partir da última posição

	DelayMs(2000);								// aguarda
	timerTIM2_init(1);							// ativa timer 1 com int

	lcd16x2_clrscr();
	lcd16x2_gotoxy(0,0);						// posiciona na linha 1, coluna 0
	lcd16x2_puts("T>   Q>   C>    ");			// escreve a partir da última posição
	lcd16x2_gotoxy(0,1);						// posiciona na linha 2, coluna 0
	lcd16x2_puts("->              ");			// escreve a partir da última posição

	while(TRUE){
		if(autoriza){							// tem dado??
			if(valida(&RxTerm[0], &dados[0])){	// testa protocolo - valida!
				envio = 1;						// avisa que vai iniciar transmissão
				lcd16x2_gotoxy(2,0);			// posiciona na linha 1, coluna 4
				bin_hex(RxTerm[0], &mhigh, &mlow);
				p[0] = mhigh;
				p[1] = mlow;
				p[2] = '\0';
				lcd16x2_puts(p);				// escreve nr do terminal que enviou mensagem

				lcd16x2_gotoxy(7,0);			// posiciona na linha 1, coluna 10
				bin_hex(RxTerm[1], &mhigh, &mlow);
				p[0] = mhigh;
				p[1] = mlow;
				p[2] = '\0';
				lcd16x2_puts(p);				// escreve quantidade de bytes recebidos

				msgcounter++;
				if(msgcounter > 999)
					msgcounter = 0;

				sprintf(p, "%03d", msgcounter);
				lcd16x2_gotoxy(12,0);			// cursor na linha 1, coluna 12
				lcd16x2_puts(p);				// envia

				lcd16x2_gotoxy(2,1);			// posiciona na linha 2, coluna 2
				lcd16x2_puts("              "); // apaga linha
				lcd16x2_gotoxy(2,1);			// posiciona na linha 2, coluna 2
				lcd16x2_puts(dados);			// escreve o que foi enviado

				TxRxbuf[0] = HEADER;			// devolve ACK
#ifdef __DEBUG__
				uint8_t aux = NR_DISP;
				aux <<= 0x04;
				aux |= RxTerm[0];
				bin_hex(aux, &mhigh, &mlow);
				TxRxbuf[1] = mhigh;
				TxRxbuf[2] = mlow;
				TxRxbuf[3] = 0x01+debug;		// quantidade de bytes
				TxRxbuf[4] = ACK;				// envia ACK
				TxRxbuf[5] = '\0';				// fecha para inserir dados
				TxRxbuf[6] = CR;
				TxRxbuf[7] = LF;
				TxRxbuf[8] = '\0';				// fecha string
#else
				TxRxbuf[1] = NR_DISP;			// número do dispositivo que envia
				TxRxbuf[1] <<= 0x04;			// posiciona no MSB
				TxRxbuf[1] |= RxTerm[0];		// número do dispositivo que receberá
				TxRxbuf[2] = 0x01;				// quantidade de bytes
				TxRxbuf[3] = ACK;				// envia ACK
				TxRxbuf[4] = '\0';				// fecha string
#endif

				for(uint8_t i=0; i<strlen(TxRxbuf);i++)//envia dados - ACK
					usart3_putchar(TxRxbuf[i]);

				envio = 0;								// avisa que está pronto para receber novamente!
			}
			autoriza = 0;						// autoriza novo buffer
		}
		else{									// analisa chaves para enviar algo
			toTerm = getBTNs();					// verifica se tem chave e envia dado
			if(toTerm){
				if(!prima){						// se esta recebendo, cancela envio!
					envio = 1;					// avisa que vai iniciar transmissão
					timeout_aut = 0;			// desligta time out
					mytimeout = 0;
					TxRxbuf[0] = HEADER;				    //
#ifdef __DEBUG__
					uint8_t aux = NR_DISP;
					aux <<= 0x04;
					aux |= toTerm;
					bin_hex(aux, &mhigh, &mlow);
					TxRxbuf[1] = mhigh;
					TxRxbuf[2] = mlow;
					TxRxbuf[3] = strlen("PE-STM32")+debug;	// quantidade de caracteres
					TxRxbuf[4] = '\0';
					strcat(TxRxbuf,"PE-STM32");
					TxRxbuf[4+strlen("PE-STM32")-1] = CR;
					TxRxbuf[5+strlen("PE-STM32")-1] = LF;
					TxRxbuf[6+strlen("PE-STM32")-1] = '\0';
#else
					TxRxbuf[1] = NR_DISP;					// número do dispositivo que envia
					TxRxbuf[1] <<= 0x04;					// posiciona no MSB
					TxRxbuf[1] |= toTerm;					// número do dispositivo que receberá
					TxRxbuf[2] = strlen("PE-STM32");	    // quantidade de caracteres
					TxRxbuf[3] = '\0';						// fecha para inserir dados
					strcat(TxRxbuf,"PE-STM32");				// insere MSG
#endif

					for(uint8_t i=0; i<strlen(TxRxbuf);i++) //envia dados
						usart3_putchar(TxRxbuf[i]);

					envio = 0;								// avisa que está pronto para receber novamente!
				}
			}
		}
	}
}
